ToggleableWiperautomatic = {}



function ToggleableWiperautomatic.prerequisitesPresent(specializations)

	return SpecializationUtil.hasSpecialization(Wipers, specializations)

end



function ToggleableWiperautomatic.registerOverwrittenFunctions(vehicleType)

	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getIsActiveForWipers", ToggleableWiperautomatic.getIsActiveForWipers)

end



function ToggleableWiperautomatic.registerEventListeners(vehicleType)

	local specFunctions = { "onLoad", "saveToXMLFile", "onRegisterActionEvents", "onReadStream", "onWriteStream", "onReadUpdateStream", "onWriteUpdateStream" }
	
	for _, specFunction in ipairs(specFunctions) do
		SpecializationUtil.registerEventListener(vehicleType, specFunction, ToggleableWiperautomatic)
	end

end



function ToggleableWiperautomatic:onLoad(savegame)

	if self.spec_ToggleableWiperautomatic == nil then
		self.spec_ToggleableWiperautomatic = {}
	end

	local spec = self.spec_ToggleableWiperautomatic
	spec.actionEvents = {}
	spec.activated = true

	if self.spec_ManualWipers ~= nil then
		spec.activated = false
	end

	if savegame ~= nil then

		local Zustand = getXMLBool(savegame.xmlFile, savegame.key..".ToggleableWiperautomatic#automatic")

		if Zustand ~= nil then
			spec.activated = Zustand
		end

	end

end



function ToggleableWiperautomatic:saveToXMLFile(xmlFile, key)

	local spec = self.spec_ToggleableWiperautomatic

	setXMLBool(xmlFile, key.."#automatic", spec.activated)

end



function ToggleableWiperautomatic:onRegisterActionEvents(isActiveForInput)

	if self.isClient then

		local spec = self.spec_ToggleableWiperautomatic

		self:clearActionEventsTable(spec.actionEvents)
		spec.actionEventIds = {}

		if self:getIsActiveForInput(true, true) then

			local _, actionEventId = self:addActionEvent(spec.actionEvents, "CHI_ScheibenwischerAutomatikEinAus", self, ToggleableWiperautomatic.onInputAction, false, true, false, true, nil)

			g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERY_LOW)
			g_inputBinding:setActionEventTextVisibility(actionEventId, false)

			table.insert(spec.actionEventIds, actionEventId)

		end

	end

end



function ToggleableWiperautomatic:onReadStream(streamId, connection)

	self.spec_ToggleableWiperautomatic.activated = streamReadBool(streamId)

end



function ToggleableWiperautomatic:onWriteStream(streamId, connection)

	streamWriteBool(streamId, self.spec_ToggleableWiperautomatic.activated)

end



function ToggleableWiperautomatic:onReadUpdateStream(streamId, timestamp, connection)

	if connection.isServer then

		self.spec_ToggleableWiperautomatic.activated = streamReadBool(streamId)

	end

end



function ToggleableWiperautomatic:onWriteUpdateStream(streamId, connection, dirtyMask)

	if not connection.isServer then

		streamWriteBool(streamId, self.spec_ToggleableWiperautomatic.activated)

	end

end



function ToggleableWiperautomatic.onInputAction(self, actionName, inputValue, callbackState, isAnalog)

	local spec = self.spec_ToggleableWiperautomatic

	if actionName == "CHI_ScheibenwischerAutomatikEinAus" then

		spec.activated = not spec.activated

	end

	if g_server ~= nil then

		g_server:broadcastEvent(ToggleableWiperautomaticEvent:new(self, spec.activated), nil, nil, self)

	else

		g_client:getServerConnection():sendEvent(ToggleableWiperautomaticEvent:new(self, spec.activated))

	end

end



function ToggleableWiperautomatic:getIsActiveForWipers(superFunc)

	if not self.spec_ToggleableWiperautomatic.activated then
		return false
	end

	return superFunc(self)

end




-- Events

ToggleableWiperautomaticEvent = {}
ToggleableWiperautomaticEvent_mt = Class(ToggleableWiperautomaticEvent, Event)

InitEventClass(ToggleableWiperautomaticEvent, "ToggleableWiperautomaticEvent")



function ToggleableWiperautomaticEvent:emptyNew()

    local self = Event:new(ToggleableWiperautomaticEvent_mt)

    self.className="ToggleableWiperautomaticEvent"

    return self

end



function ToggleableWiperautomaticEvent:new(vehicle, activated)

	local self = ToggleableWiperautomaticEvent:emptyNew()

	self.vehicle = vehicle
	self.activated = activated

	return self

end



function ToggleableWiperautomaticEvent:readStream(streamId, connection)

	self.vehicle = NetworkUtil.readNodeObject(streamId)
	self.activated = streamReadBool(streamId)

	self:run(connection)

end



function ToggleableWiperautomaticEvent:writeStream(streamId, connection)

	NetworkUtil.writeNodeObject(streamId, self.vehicle)

	streamWriteBool(streamId, self.activated)

end



function ToggleableWiperautomaticEvent:run(connection)

	self.vehicle.spec_ToggleableWiperautomatic.activated = self.activated

	if self.vehicle ~= nil then

		if not connection:getIsServer() then

			g_server:broadcastEvent(ToggleableWiperautomaticEvent:new(self.vehicle, self.activated), nil, connection, self.vehicle)

		end

	end

end



function ToggleableWiperautomaticEvent.sendEvent(vehicle, activated, noEventSend)

	if noEventSend == nil or noEventSend == false then

		if g_server ~= nil then

			g_server:broadcastEvent(ToggleableWiperautomaticEvent:new(vehicle, activated), nil, nil, vehicle)

		else

			g_client:getServerConnection():sendEvent(ToggleableWiperautomaticEvent:new(vehicle, activated))

		end

	end

end